#!/bin/bash
#Description: plugin agent uninstall entry
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2024. All rights reserved.
#History: 2024-8-12 created

current_dir=$(cd $(dirname "$0"); pwd)
source "${current_dir}/utils.sh"
install_path="$1/DevKit"
DISK_PATH="/devkit_sysmig_disk"


# 删除插件安装目录
function delete_porting()
{
    echo_and_log "Delete porting install path" "${DOING}" "${LOG}"
    local porting_path="${install_path}/devkitplugins/porting"
    if [[ -d "${porting_path}" ]]; then
        rm -rf "${porting_path}"
    fi
    echo_and_log "Delete porting install path success" "${SUCCESS}" "${LOG}"
}

function plugin_clean()
{
    local devkitclient_pid
    devkitclient_pid=$(ps -ef | grep "tools/rpc_cluster/client/bin/devkitclient" | grep -v grep | awk '{print $2}')
    local kill_process="${install_path}/devkitplugins/porting/deploy/kill_process.sh"
    if [[ -n "${devkitclient_pid}" ]] && [[ -f "${kill_process}" ]]; then
        bash "${kill_process}" "${devkitclient_pid}" 15 >> /dev/null 2>&1
    fi
}

function umount_disk()
{
    echo_and_log "umount disk" "${DOING}" "${LOG}"
    if [[ -f "${DISK_PATH}/disk_info.txt" ]]; then
        for mount_point in `cat ${DISK_PATH}/disk_info.txt`
        do
            if [[ "$(lsblk -a | grep -E "${mount_point}$" | awk -F' ' '{print $7}')" == "${mount_point}" ]]; then
                echo_and_log "umount ${mount_point}" "${ECHO}" "${LOG}"
                umount ${mount_point}
            fi
            if [[ -d ${mount_point} ]] && [[ -z "$(ls -A ${mount_point})" ]]; then
                echo_and_log "rmdir ${mount_point}" "${ECHO}" "${LOG}"
                rmdir ${mount_point}
            fi
        done
        rm -f "${DISK_PATH}/disk_info.txt"
    fi
    if [[ "$(lsblk -a | grep -E "${DISK_PATH}$" | awk -F' ' '{print $7}')" == "${DISK_PATH}" ]]; then
        echo_and_log "umount ${DISK_PATH}" "${ECHO}" "${LOG}"
        umount ${DISK_PATH}
    fi
    if [[ -d ${mount_point} ]] && [[ -z "$(ls -A ${DISK_PATH})" ]]; then
        echo_and_log "rmdir ${DISK_PATH}" "${ECHO}" "${LOG}"
        rmdir ${DISK_PATH}
    fi
    echo_and_log "Delete porting install path success" "${SUCCESS}" "${LOG}"
}

function main()
{
    LOG="${install_path}/logs/porting_uninstall.log"
    if [[ -f "${LOG}" ]]; then
        rm -f "${LOG}"
    fi
    touch "${LOG}"
    chmod 600 "${LOG}"
    echo_and_log "Uninstalling porting agent" "${DOING}" "${LOG}"
    plugin_clean
    umount_disk
    x2openEuler_deploy "${LOG}" "uninstall"
    delete_sudo_tools "${LOG}"
    delete_porting
    echo_and_log "Uninstalling porting agent success" "${SUCCESS}" "${LOG}"
}

main $@
