#!/bin/bash
#Description: DevKit constants
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-2-9 created
current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/const.sh"
source "${current_dir}/utils.sh"
architecture=$(uname -m)
PLUGIN="porting"
JDK_DIR_NAME=jre-17.0.13

function init_variable() {
	if [ ! -f "${INSTALL_PATH}/tools/log.sh" ]; then
		echo "There is no log file(${INSTALL_PATH}/tools/log.sh), please check!"
		exit 1
	fi
	source ${INSTALL_PATH}/tools/log.sh
	INSTALL_INI_PATH="${INSTALL_PATH}/config/install.ini"
	if [[ ! -f "${INSTALL_INI_PATH}" ]]; then
		echo_and_log "There is no configure file(${INSTALL_INI_PATH}), please check!" "${ERROR}"
		exit "${ERROR}"
	fi
	USER_PATH="$(grep -w USER_PATH "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
	USER_NAME="$(grep -w USER_NAME "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')"
	gunicorn_ip=$(grep -w GUNICORN_IP "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	gunicorn_port=$(grep -w PLUGIN_GUNICORN_PORT "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	os_type=$(grep -w OS_TYPE "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	os_version=$(grep -w OS_VERSION "${INSTALL_INI_PATH}" | awk -F= '{if($2 !="") print $2}')
	plugin_name="porting"
	INSTALL_LOG_PATH="${INSTALL_PATH}/logs/install_plugins.log"
	INSTALL_PLUGIN_PATH="${INSTALL_PATH}/devkitplugins"
	# 插件压缩包存档路径
	PYTHON_PATH="${INSTALL_PATH}/tools/python_venv/bin/python3"

	DEPENDENCY_DICTIONARY_PATH="${INSTALL_PATH}/devkitplugins/porting/config/restore_dependency_package"
	dependency_dictionary_name=$(ls ${DEPENDENCY_DICTIONARY_PATH} | grep "Dependency-Dictionary")
	MIGRATION_PATH="${INSTALL_PATH}/devkitplugins/porting/config/restore_dependency_package"
	migration_name=$(ls ${MIGRATION_PATH} | grep "Migration-Package")
}

function change_file_permission() {
	local file="$1"
	local num="$2"
	if [ -f "${file}" ]; then
		chmod "${num}" "${file}"
	fi
}

function install_dependency_dictionary_status() {
	local sqlite3_bin_path=$(which sqlite3)
	local sqlite3_file_path="${INSTALL_PATH}/devkitplugins/sqlite/db.sqlite3"
	${sqlite3_bin_path} "${sqlite3_file_path}" "insert into dependency_dictionary_dependencydictionary (file_name, file_source) values ('${dependency_dictionary_name}', 0);"
}

function change_file_path_permission() {
	# 修改文件的权限
	all_asm_path=${INSTALL_PATH}/devkitplugins/${plugin_name}/all_asm
	inline_asm_path=${INSTALL_PATH}/devkitplugins/${plugin_name}/inline_asm
	if [ -d ${all_asm_path} ] && [ -d ${inline_asm_path} ]; then
		chmod -R 500 ${all_asm_path}/bin
		chmod 600 ${all_asm_path}/config/AllAsmLogCfg.json
		chmod 400 ${all_asm_path}/config/dependency.conf
		chmod -R 400 ${all_asm_path}/lib
		chmod 700 ${all_asm_path}/bin ${all_asm_path}/lib ${all_asm_path}/lib/semantics

		chmod -R 500 ${inline_asm_path}/clang-tidy/bin/* ${inline_asm_path}/clang-tidy/lib/* ${inline_asm_path}/remill/bin/* ${inline_asm_path}/remill/lib/*
		chmod -R 400 ${inline_asm_path}/clang-tidy/include/* ${inline_asm_path}/remill/lib/semantics/*
		chmod -R 600 ${inline_asm_path}/config/*
		chmod 700 $(find ${inline_asm_path}/clang-tidy/include/ -type d)
		chmod 700 ${inline_asm_path}/remill/lib/semantics/
	fi

	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/decode_gcm.so" 400
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/log.conf" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/log/${plugin_name}.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/porting_rules.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/ignore_rules.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/migration_route.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/mapping_rules.json" 600
	change_file_permission "${INSTALL_PATH}/devkitplugins/${plugin_name}/config/download.conf" 600

	change_file_permission "${DEPENDENCY_DICTIONARY_PATH}/${dependency_dictionary_name}" 600
	change_file_permission "${MIGRATION_PATH}/${migration_name}" 600
	chmod 600 $(find ${INSTALL_PATH}/devkitplugins/${plugin_name}/config/sysmig -type f)
	chmod 600 $(find ${INSTALL_PATH}/devkitplugins/${plugin_name}/sysmig -name "*.json")
	chmod 600 $(find ${INSTALL_PATH}/devkitplugins/${plugin_name}/config/xc_model -type f)
	chmod 600 $(find ${INSTALL_PATH}/devkitplugins/${plugin_name}/sysmig/tool/schema/json_schema_file -type f)
	chmod 700 $(find ${INSTALL_PATH}/devkitplugins/porting/resource/migration -type f)
	chmod 600 $(find ${INSTALL_PATH}/devkitplugins/porting/resource/migration -regex ".*\.cipher")
}

function insert_to_sudoers() {
	# 暂时只支持Kunpeng, CentOS7.6和openEuler20.03sp1上进行解决方案移植
	if [[ $(uname -m) == "aarch64" ]]; then
		openeuler_version=$(cat /etc/os-release | grep VERSION= | awk -F '"' '{print$2}')
		if [[ "$os_type" == "CentOS" && ${os_version} == "7.6" ]] ||
			[[ -f /etc/openEuler-release && ${openeuler_version} == "20.03 (LTS-SP1)" ]]; then
			add_devkit_plugin_sudo_list
			if [ "0" != "$?" ]; then
				exit ${ERROR}
			fi
		fi
	fi
}

function add_devkit_sudo() {
	local install_log="$3"
	if [ ! -f "${sudo_file}" ]; then
		touch $sudo_file
	fi
	chmod 440 $sudo_file
	command="$1"
	command_path="$2"
	if grep "${USER_NAME}    ALL=(root)      NOPASSWD:${command_path}" "$sudo_file"; then
		echo_and_log "${command} is already in the ${USER_NAME} sudoer list." "${DOING}" ${install_log}
	else
		# 空文件sed 追加不进去 sed 根据行号追加内容
		echo "${USER_NAME}    ALL=(root)      NOPASSWD:""${command_path}" >>"$sudo_file"
	fi
}

function modify_spec_default_value() {
	# 修改专项软件迁移中spec以及sh 文件中的写死的/home/devkit
	migration_path="${INSTALL_PATH}/devkitplugins/porting/resource/migration"
	if [[ "$(grep -r "/home/devkit" ${migration_path})" ]]; then
		grep -r "/home/devkit" ${migration_path} | awk -F ":" '{print $1}' | xargs sed -i "s#/home/devkit#${USER_PATH}#"g
	fi
}

function add_devkit_plugin_sudo_list() {
	centos_parameter_list=()
	openEuler_parameter_list=()
	if [[ -f /etc/openEuler-release ]]; then
		local parameter_list_len=${#openEuler_parameter_list[@]}
	else
		local parameter_list_len=${#centos_parameter_list[@]}
	fi
	sudo_file="/etc/sudoers.d/devkit_porting_sudoers"
	for ((i = 0; i < ${parameter_list_len}; i++)); do
		if [[ -f /etc/openEuler-release ]]; then
			local parameter=${openEuler_parameter_list[i]}
		else
			local parameter=${centos_parameter_list[i]}
		fi
		local cmd_name=$(echo ${parameter} | cut -d " " -f 1)
		add_devkit_sudo "${cmd_name}" "${parameter}" "${INSTALL_LOG_PATH}"
		if [ "0" != "$?" ]; then
			exit ${ERROR}
		fi
	done
}

function create_sysmig_lock() {
	# 安装完成后修改系统迁移锁的属组
	touch "${INSTALL_PATH}/config/lock_file/system_migration.lock"
	chown -R devkit:devkit "${INSTALL_PATH}/config/lock_file/system_migration.lock"
}

function build_agent_plugins() {
	# 定义一个数组，包含需要安装到节点的文件
	local package_files=("cmd" "deploy/kill_process.sh" "deploy/agent_install_entry.sh" "deploy/agent_uninstall_entry.sh" "deploy/const.sh" "deploy/agent_env_check.sh" "deploy/utils.sh")
	# 定义一个数组，包含需要从插件安装包中删除的文件
	local plugin_delete_files=()
	local framework_install_ini_path="${INSTALL_PATH}/config/install.ini"
	if [[ -f "${framework_install_ini_path}" ]]; then
		local framework_version="$(grep -w FRAMEWORK_VERSION "${framework_install_ini_path}" | awk -F= '{if($2 !="") print $2}')"
		plugin_delete_files+=("cmd/bin/sql-analysis-${framework_version}.jar")
	fi
	# 定义插件目录
	local plugin_dir="${INSTALL_PATH}/devkitplugins/${PLUGIN}"

	# 定义插件包目录
	local plugin_package_dir="${INSTALL_PATH}/tools/agent_install_packages/${PLUGIN}"
	# 如果插件包目录存在，则删除该目录
	if [[ -d "${plugin_package_dir}" ]]; then
		rm -rf "${plugin_package_dir}"
	fi
	# 创建插件包目录，并设置权限
	mkdir "${plugin_package_dir}"
	chmod 700 "${plugin_package_dir}"

	# 如果插件目录不存在，则输出错误日志并退出
	if [[ ! -d "${plugin_dir}" ]]; then
		echo_and_log "Plugin ${PLUGIN} not exists" "${ERROR}" "${INSTALL_LOG_PATH}"
		exit "${ERROR}"
	else
		# 遍历需要安装的文件
		for package_file in "${package_files[@]}"; do
			# 定义源文件路径和目标文件路径
			local src_file="${plugin_dir}/${package_file}"
			local dist_file="${plugin_package_dir}/${package_file}"
			local dist_dir
			dist_dir="$(dirname ${dist_file})"
			# 如果目标文件路径的父目录不存在，则创建父目录
			if [[ ! -d "${dist_dir}" ]]; then
				mkdir -p "${dist_dir}"
			fi
			# 如果源文件是目录，则递归复制
			if [[ -d "${src_file}" ]]; then
				if ! cp -rp -P --remove-destination "${src_file}" "${dist_file}"; then
					echo_and_log "Copy ${src_file} to ${dist_file} failed" "${ERROR}" "${INSTALL_LOG_PATH}"
					exit "${ERROR}"
				fi
			# 如果源文件是文件，则复制文件
			elif [[ -f "${src_file}" ]]; then
				if ! cp -p -P --remove-destination "${src_file}" "${dist_file}"; then
					echo_and_log "Copy ${src_file} to ${dist_file} failed" "${ERROR}" "${INSTALL_LOG_PATH}"
					exit "${ERROR}"
				fi
			fi
		done
	fi

	# 先删掉多余文件
	if [[ ! -d "${plugin_package_dir}" ]]; then
		echo_and_log "Pre-install Plugin ${PLUGIN} not exists" "${ERROR}" "${INSTALL_LOG_PATH}"
		exit "${ERROR}"
	else
		if [[ ! -z "${plugin_delete_files}" ]]; then
			# 遍历需要删除的文件
			for delete_file in "${plugin_delete_files[@]}"; do
				local delete_item="${plugin_package_dir}/${delete_file}"
				# 如果是目录并且不是链接，则删除目录
				if [[ -d "${delete_item}" ]] && [[ ! -L "${delete_item}" ]]; then
					rm -rf "${delete_item}"
				# 如果是文件并且不是链接，则删除文件
				elif [[ -f "${delete_item}" ]] && [[ ! -L "${delete_item}" ]]; then
					rm -f "${delete_item}"
				fi
				# 如果删除失败，则输出错误日志并退出
				if [[ $? -ne 0 ]]; then
					echo_and_log "Delete ${delete_item} failed" "${ERROR}" "${INSTALL_LOG_PATH}"
					exit "${ERROR}"
				fi
			done
		fi
	fi
	# 定义tar包目录和插件目录
	local tar_dir
	tar_dir="$(dirname "${plugin_package_dir}")" # 获取变量目录名
	local plugins_dir
	plugins_dir="$(basename "${plugin_package_dir}")" # 获取变量文件名
	# 定义tar包路径
	local plugins_tar_path="${tar_dir}/${PLUGIN}.tar.gz"
	# 如果tar包已经存在，则删除
	if [[ -f "${plugins_tar_path}" ]]; then
		rm -f "${plugins_tar_path}"
	fi
	# 创建tar包
	tar -czf "${plugins_tar_path}" -C "${tar_dir}" "${plugins_dir}"
	# 如果创建成功，则删除插件包目录，并设置文件权限
	if [[ $? -eq 0 ]]; then                       # 检查上一个命令是否成功执行
		cd "${tar_dir}" && rm -rf "./${plugins_dir}" # 把复制过来的文件删了，只留下压缩包
		chown root:devkit "${plugins_tar_path}"
		chmod 440 "${plugins_tar_path}"
		# 生成文件签名
		local sign_for_digest
		sign_for_digest=$(su - "devkit" -s /bin/bash -c "${PYTHON_PATH} -c \"import sys;sys.path.append('${INSTALL_PATH}/devkitplugins');from devkitcommon.common_utils.common_util import get_file_hash, sign_for_digest;print(sign_for_digest(get_file_hash('${plugins_tar_path}')))\"")
		# 如果签名存在，则创建签名文件，并设置文件权限
		if [[ -n "${sign_for_digest}" ]]; then
			local plugins_sign_path="${tar_dir}/${PLUGIN}.sign"
			echo "${sign_for_digest}" >"${plugins_sign_path}"
			chown root:devkit "${plugins_sign_path}"
			chmod 440 "${plugins_sign_path}"
		fi
		echo_and_log "Generate plugins agent package success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
	else
		echo_and_log "Generate plugins agent package failed" "${ERROR}" "${INSTALL_LOG_PATH}"
		exit "${ERROR}"
	fi
}

# 参数2个，为插件名,和是否为接口调用 手动安装添加验签
function main() {
	# 用户通过接口调用安装
	INSTALL_PATH="$1" # /opt/DevKit
	plugin_flag="$2"  # web
	init_variable
	if [[ "${plugin_flag}" == "web" ]]; then
		bash "${current_dir}/env_check.sh" "${INSTALL_PATH}" "${plugin_flag}"
		local check_res="$?"
		chmod 444 ${current_dir}/make_env_check_result.pyc
		su - "devkit" -s /bin/bash -c "${PYTHON_PATH} ${current_dir}/make_env_check_result.pyc"
		local temp_env_json="${current_dir}/porting_env_res.json"
		local final_env_json="${INSTALL_PATH}/devkitplugins/check_env_result/porting_env_res.json"
		if [[ -f "${temp_env_json}" ]]; then
			mv "${temp_env_json}" "${final_env_json}"
			chown -h "${DEVKIT_USER}":"${DEVKIT_USER}" "${final_env_json}"
			su - "${DEVKIT_USER}" -c "chmod 600 ${final_env_json}"
		fi
		if [[ "${check_res}" != "${SUCCESS}" ]]; then
			echo_and_log "Check install Dependent Packages Failed" "${ERROR}" "${INSTALL_LOG_PATH}"
			exit "${ERROR}"
		else
			echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
		fi
	else
		# 磁盘空间大小检测
		echo_and_log "Check disk space" "${DOING}" "${INSTALL_LOG_PATH}"
		disk_space_check "${INSTALL_PATH}" "${INSTALL_SPACE_NEED}"
		if [ "$?" -ne "${SUCCESS}" ]; then
			echo_and_log "Check install path disk space Failed" "${ERROR}" "${INSTALL_LOG_PATH}"
			exit "${ERROR}"
		fi
		echo_and_log "Check install path disk space Success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
	fi

	insert_to_sudoers
	change_file_path_permission
	create_sysmig_lock
	install_dependency_dictionary_status
	modify_spec_default_value
	# 删除依赖检查配置文件
	if [[ -e "${INSTALL_PATH}/devkitplugins/${plugin_name}/deploy/porting_env_res.ini" ]]; then
		rm -rf "${INSTALL_PATH}/devkitplugins/${plugin_name}/deploy/porting_env_res.ini"
	fi
	if [ "${architecture}" == "aarch64" ]; then
		JDK_PACKAGE_NAME=${JDK_DIR_NAME}-linux-aarch64.tar.gz
		tar -zxf ${INSTALL_PATH}/devkitplugins/porting/${JDK_PACKAGE_NAME} -C ${INSTALL_PATH}/devkitplugins/porting/cmd/bin
		chown -hR devkit:devkit ${INSTALL_PATH}/devkitplugins/porting/cmd/bin/${JDK_DIR_NAME}
		su - devkit -s /bin/bash -c "chmod -R o-rwx ${INSTALL_PATH}/devkitplugins/porting/cmd/bin/${JDK_DIR_NAME}"
		#解压新的jdk
		tar -zxf ${INSTALL_PATH}/devkitplugins/porting/${JDK_PACKAGE_NAME} -C ${INSTALL_PATH}/devkitplugins/porting
		chown -hR devkit:devkit ${INSTALL_PATH}/devkitplugins/porting/${JDK_DIR_NAME}
		su - devkit -s /bin/bash -c "chmod -R o-rwx ${INSTALL_PATH}/devkitplugins/porting/${JDK_DIR_NAME}"
		rm ${INSTALL_PATH}/devkitplugins/porting/${JDK_PACKAGE_NAME}
	else
		JDK_PACKAGE_NAME=${JDK_DIR_NAME}-linux-x64.tar.gz
	fi

	if [ "${architecture}" == "aarch64" ]; then
		build_agent_plugins
	fi
	chmod -R o-rwx ${INSTALL_PATH}/devkitplugins/porting/cmd/
}

main $@
