#!/bin/bash
#Description: const params
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2020. All rights reserved.
#History: 2020-05-11 created

current_dir=$(cd $(dirname $0); pwd)
source "${current_dir}/const.sh"

# 获取系统磁盘空间，检查磁盘空间是否满足安装
function disk_space_check()
{
    local install_path=$1
    local disk_space_need=$2
    local install_path_parent=$(dirname "${install_path}")
    # 获取磁盘的百分之十的大小，与disk_space_need比较
    local total_size=$(df "${install_path_parent}" | awk 'NR==2' | awk '{print $2}')
    local percent_size=$(expr ${total_size} / 10)
    if [[ ${percent_size} -gt ${TOOL_USE_SPACE} ]]; then
        disk_space_need=`expr ${TOOL_USE_SPACE} + ${disk_space_need}`
    else
        disk_space_need=`expr ${percent_size} + ${disk_space_need}`
    fi

    local available_space=$(df "$install_path_parent" | awk 'NR==2' | awk '{print $4}')
    if [[ ${available_space} -lt ${disk_space_need} ]]; then
        print_disk_space_need=`expr ${disk_space_need} / 1000 / 1000`
        echo_and_log "Insufficient drive space. At least ${print_disk_space_need} GB free space is required. Release drive space and try again." "${ERROR}"
        return "${ERROR}"
    fi
    return "${SUCCESS}"
}

# 终端及日志打印:echo_and_log [info] [status] [log]
function echo_and_log()
{
    local info=$1
    local status=$2
    local log=""
    local type="32"
    if [[ $# == 3 ]]; then
        log=$3 # 选填
        if [[ ! -f "${log}" ]]; then
            echo -e "\e[1;31m echo_and_log:${log} is not exist \e[0m"
            exit "${ERROR}"
        fi
    fi

    case "${status}" in
        0)
            type="32";;
        1)
            type="31";;
        2)
            type="33";;
        3)
            type="37";; # 白色
        *)
            echo -e "\e[1;31m status: 0/1/2, input is ${status} \e[0m";;
    esac
    if [[ -z "${log}" ]]; then
        echo -e "\e[1;${type}m  ${info} \e[0m"
    else
        if [[ ! -f "${log}" ]]; then
            echo -e "\e[1;31m echo_and_log:${log} is not exist \e[0m"
            exit "${ERROR}"
        fi
        echo -e "\e[1;${type}m  ${info} \e[0m" | tee -a "${log}"
    fi
}

function delete_sudo_tools()
{
    local LOG=$1
    echo_and_log "Delete sudo tools" "${DOING}" "${LOG}"
    if [[ -f "${SUDO_FILE}" ]]; then
        rm -f "${SUDO_FILE}"
    fi
    echo_and_log "Delete sudo tools success" "${SUCCESS}" "${LOG}"
}

function x2openEuler_deploy()
{
    local LOG=$1
    local mode=$2
    local x2openEuler_install_deploy="${install_path}/devkitplugins/porting/cmd/x2openEuler_install/deploy-x2openEuler"
    local x2openEuler_rpm_path="${install_path}/devkitplugins/porting/cmd/x2openEuler_install/rpm"
    if [[ -d "${x2openEuler_rpm_path}" ]]; then
        local x2openEuler_install_rpm="$(find "${x2openEuler_rpm_path}" -type f -name "*.aarch64.rpm" -print -quit)"
    fi
    if [[ -f "${x2openEuler_install_deploy}" && -n "${x2openEuler_install_rpm}" ]]; then
        bash "${x2openEuler_install_deploy}" "-p" "${x2openEuler_install_rpm}" "${mode}"
        if [[ $? -ne "0" ]]; then
            echo_and_log "Failed to ${mode} x2openEuler" "${ERROR}" "${LOG}"
        else
            echo_and_log "Successfully ${mode}ed x2openEuler" "${SUCCESS}" "${LOG}"
        fi
    else
        echo_and_log "Failed to find x2openEuler deployment file" "${ERROR}" "${LOG}"
    fi
}